@@ -1,7 +1,11 @@
 package org.gcube.applicationsupportlayer.social.mailing;
 
+import java.text.Format;
+import java.text.SimpleDateFormat;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.Date;
+import java.util.List;
 
 import javax.mail.Message;
 import javax.mail.Multipart;
@@ -12,6 +16,10 @@ import javax.mail.internet.MimeMessage;
 import javax.mail.internet.MimeMultipart;
 
 import org.gcube.portal.custom.communitymanager.OrganizationsUtil;
+import org.gcube.portal.databook.server.DBCassandraAstyanaxImpl;
+import org.gcube.portal.databook.server.DatabookStore;
+import org.gcube.portal.databook.shared.Comment;
+import org.gcube.portal.databook.shared.Feed;
 import org.gcube.portal.databook.shared.Notification;
 import org.gcube.portal.databook.shared.NotificationType;
 import org.jsoup.Jsoup;
@@ -35,9 +43,10 @@ public class EmailPlugin {
 
 	private static final Logger _log = LoggerFactory.getLogger(EmailPlugin.class);
 	protected static final int SECONDS2WAIT = 60;
-	public static final String WRITE_ABOVE_TO_REPLY = "- Write ABOVE THIS LINE to post a reply via email, reply with empty msg to subscribe -";
-
+	public static final String WRITE_ABOVE_MESSAGE_REPLY = "- Write ABOVE THIS LINE to reply via email";
+	public static final String WRITE_ABOVE_TO_REPLY = WRITE_ABOVE_MESSAGE_REPLY + ", reply with empty msg to subscribe -";
 	private static EmailPlugin singleton;
+	private static DatabookStore store = new DBCassandraAstyanaxImpl();
 
 	public static EmailPlugin getInstance() {
 		if (singleton == null)
@@ -51,12 +60,16 @@ public class EmailPlugin {
 
 	protected static ArrayList<NotificationMail> BUFFER_EMAILS = new ArrayList<NotificationMail>();
 
-	private static String getHTMLEmail(Notification notification2Save, String userFirstName, String portalURL, String email, String ... hashtags) {
-
+	private static String getHTMLEmail(Notification notification2Save, 
+			String userFirstName, 
+			String portalURL, 
+			String email, 
+			String vreName, 
+			Feed feed,
+			List<Comment> comments,
+			String commentKey,
+			String ... hashtags) {
 		String removedMarkup = notification2Save.getDescription().replaceAll("&amp;", "&");
-		removedMarkup = removedMarkup.replaceAll("&gt;", ">");
-		removedMarkup = removedMarkup.replaceAll("&lt;", "<");
-
 		if (hashtags != null && hashtags.length > 0) {
 			_log.debug("editing hyperlinks for mail client");
 			//notification2Save uri contains the absoulte path to the feed in the correct channel, e.g. /group/ustore_vre?oid=f1637958-34d0-48fc-b5ad-13b1116e389d
@@ -83,13 +96,20 @@ public class EmailPlugin {
 				notification2Save.getType() == NotificationType.MENTION || 
 				notification2Save.getType() == NotificationType.OWN_COMMENT) {
 			body.append("<div>").append(WRITE_ABOVE_TO_REPLY).append("</div><br />");
+
+
+		}
+		if (notification2Save.getType() == NotificationType.MESSAGE) {
+			body.append("<div>").append(WRITE_ABOVE_MESSAGE_REPLY).append("</div><br />");
 		}
 
 		body.append("<div style=\"color:#000; font-size:13px; font-family:'lucida grande',tahoma,verdana,arial,sans-serif;\">")
 		.append("Dear ").append(userFirstName).append(",")  //dear <user>
 		.append("<p>").append(sender).append(" ").append(removedMarkup) // has done something
-		.append(getActionLink(notification2Save, portalURL)).append("</p>")  //Goto
-		.append("</div><br />")
+		.append(getActionLink(notification2Save, portalURL)) //Goto
+		.append(buildHtmlDiscussion(notification2Save, feed, comments, commentKey)) // the original discussion
+		.append("</p>")  
+		.append("</div><br/>")
 		.append("<p><div style=\"color:#999999; font-size:11px; font-family:'lucida grande',tahoma,verdana,arial,sans-serif;\">")
 		.append("This message was sent to <a href=\"mailto:")
 		.append(email)
@@ -100,7 +120,7 @@ public class EmailPlugin {
 		.append(portalURL).append("/group/data-e-infrastructure-gateway/notifications\" style=\"color:#3b5998;text-decoration:none\" target=\"_blank\">unsubscribe</a>.")
 		.append("</div></p>")
 		.append("<p><div style=\"color:#999999; font-size:10px; font-family:'lucida grande',tahoma,verdana,arial,sans-serif; padding-top:15px;\">")
-		.append("WARNING / LEGAL TEXT: This message is intended only for the use of the individual or entity to which it is addressed and may contain")
+		.append("WARNING / LEGAL TEXT: This message is intended only for the use of the individual or entity to which it is addressed and may contain ")
 		.append("information which is privileged, confidential, proprietary, or exempt from disclosure under applicable law. If you are not the intended recipient or the person responsible for delivering the message to the intended recipient, you are strictly prohibited from disclosing, distributing, copying, or in any way using this message.")
 		.append("If you have received this communication in error, please notify the <sender> and destroy and delete any copies you may have received.")
 		.append("</div></p>")
@@ -110,7 +130,15 @@ public class EmailPlugin {
 
 	}
 
-	private static String getTextEmail(Notification notification2Save, String userFirstName, String portalURL, String email, String[] hashtags) {
+	private static String getTextEmail(
+			Notification notification2Save, 
+			String userFirstName, 
+			String portalURL, 
+			String email, 
+			Feed feed,
+			List<Comment> comments,
+			String commentKey,
+			String[] hashtags) {
 
 		String removedMarkup = convertHTML2Text(notification2Save.getDescription());		
 
@@ -131,17 +159,23 @@ public class EmailPlugin {
 
 		StringBuilder body = new StringBuilder();
 
+
 		if (notification2Save.getType() == NotificationType.POST_ALERT || 
 				notification2Save.getType() == NotificationType.COMMENT || 
 				notification2Save.getType() == NotificationType.MENTION || 
 				notification2Save.getType() == NotificationType.OWN_COMMENT) {
 			body.append(WRITE_ABOVE_TO_REPLY).append("\n\n");
+
+		}
+		if (notification2Save.getType() == NotificationType.MESSAGE) {
+			body.append(WRITE_ABOVE_MESSAGE_REPLY).append("\n\n");
 		}
 
 		body.append("Dear ").append(userFirstName).append(",")  //dear <user>
 		.append("\n").append(sender).append(" ").append(removedMarkup) // has done something
 		.append("\nsee: ").append(portalURL).append(notification2Save.getUri())
-		.append("\n\n\n----\n")
+		.append(buildPlainTextDiscussion(notification2Save, feed, comments, commentKey))
+		.append("\n----\n")
 		.append("This message was sent to ")
 		.append(email)
 		.append(" by ")
@@ -153,6 +187,144 @@ public class EmailPlugin {
 
 	}
 
+	/**
+	 * Build up a discussion given the feed and its comments. 
+	 * @param notification2Save
+	 * @param feed
+	 * @param comments
+	 * @param commentKey if not null, when building the discussion stop at this comment.
+	 * @return an html string representing the discussion
+	 */
+	private static String buildHtmlDiscussion(Notification notification2Save, Feed feed, List<Comment> comments, String commentKey){
+
+		try{
+
+			if(notification2Save.getType().equals(NotificationType.POST_ALERT) || feed == null)
+				return "";
+
+			String htmlPost = "<br />" + "<br />----<p>Original post:</p>";
+			// data formatter
+			Format formatter = new SimpleDateFormat("yyyy-MM-dd hh:mm a");
+
+			// escape html
+			String feedTextNoHtml = convertHTML2Text(feed.getDescription());
+
+			// build up html post + comments
+			if(notification2Save.getType() == NotificationType.POST_ALERT || (comments.size() == 0 && notification2Save.getType() == NotificationType.MENTION))
+				htmlPost += "<div style=\"margin-top: 10px;  margin-bottom: 10px;padding-left: 15px;  "
+						+ "font-style: italic; font-weight:bold\">"
+						+ feed.getFullName() 
+						+ ": " 
+						+ (feedTextNoHtml.length() > 30 ? feedTextNoHtml.substring(0, 30) + " ..." : feedTextNoHtml) 
+						+ "<p style=\"font-family:Lucida Grande,"
+						+ "Verdana,Bitstream Vera Sans,Arial,sans-serif; "
+						+ "white-space: nowrap; font-size: smaller; color: #999;\">" + formatter.format(feed.getTime()) + "</p>"
+						+"</div>";
+			else
+				htmlPost += "<div style=\"margin-top: 10px;  margin-bottom: 10px;padding-left: 15px;  "
+						+ "font-style: italic\">"
+						+ feed.getFullName() 
+						+ ": " 
+						+ (feedTextNoHtml.length() > 30 ? feedTextNoHtml.substring(0, 30) + " ..." : feedTextNoHtml) 
+						+ "<p style=\"font-family:Lucida Grande,"
+						+ "Verdana,Bitstream Vera Sans,Arial,sans-serif; "
+						+ "white-space: nowrap; font-size: smaller; color: #999;\">" + formatter.format(feed.getTime()) + "</p>"
+						+"</div>";
+
+			for (int i = 0; i < comments.size(); i++) {
+
+				String commentTextNoHtml = convertHTML2Text(comments.get(i).getText());
+
+				if((commentKey != null && comments.get(i).getKey().equals(commentKey)) && !(notification2Save.getType() == NotificationType.POST_ALERT)){
+					htmlPost += "<div style=\"margin-top: 10px;  margin-bottom: 10px;  margin-left: 50px;  padding-left: 15px;  "
+							+ "border-left: 3px solid #ccc; font-style: italic; font-weight:bold\">"
+							+ comments.get(i).getFullName() 
+							+ ": " 
+							+ commentTextNoHtml 
+							+ "<p style=\"font-family:Lucida Grande,"
+							+ "Verdana,Bitstream Vera Sans,Arial,sans-serif; "
+							+ "white-space: nowrap; font-size: smaller; color: #999;\">" + formatter.format(comments.get(i).getTime()) + "</p>"
+							+"</div>";
+
+					break;
+				}
+				else
+					htmlPost += "<div style=\"margin-top: 10px;  margin-bottom: 10px;  margin-left: 50px;  padding-left: 15px;  "
+							+ "border-left: 3px solid #ccc; font-style: italic;\">"
+							+ comments.get(i).getFullName() 
+							+ ": " 
+							+ commentTextNoHtml 
+							+ "<p style=\"font-family:Lucida Grande,"
+							+ "Verdana,Bitstream Vera Sans,Arial,sans-serif; "
+							+ "white-space: nowrap; font-size: smaller; color: #999;\">" + formatter.format(comments.get(i).getTime()) + "</p>"
+							+"</div>";
+			} 
+
+			return htmlPost;
+		}catch(Exception e){
+			_log.error("Unable to reconstruct html discussion to put into the email body.", e);
+		}
+
+		return "";
+	}
+
+	/**
+	 * Build a plain text discussion given a feed and its comments.
+	 * @param feed
+	 * @param comments
+	 * @param commentKey if not null, when building the discussion stop at this comment.
+	 * @return a string representing the discussion
+	 */
+	private static String buildPlainTextDiscussion(Notification notification2Save, Feed feed, List<Comment> comments, String commentKey){
+
+		try{
+			
+			if(notification2Save.getType().equals(NotificationType.POST_ALERT) || feed == null)
+				return "";
+
+			// build discussion
+			String discussion = "\n\n----\n\nOriginal post:";
+
+			// data formatter
+			Format formatter = new SimpleDateFormat("yyyy-MM-dd hh:mm a");
+
+			// escape html
+			String feedTextNoHtml = convertHTML2Text(feed.getDescription());
+
+			// build up  post + comments
+			discussion += 
+					"\n"
+							+ "["  + formatter.format(feed.getTime())  + "] " 
+							+ feed.getFullName() 
+							+ ": " 
+							+ (feedTextNoHtml.length() > 30 ? feedTextNoHtml.substring(0, 30) + " ..." : feedTextNoHtml)
+							+ "\n";
+
+
+			for (int i = 0; i < comments.size(); i++) {
+
+				String commentTextNoHtml = convertHTML2Text(comments.get(i).getText());
+
+				discussion += 
+						"\t"
+								+ "["  + formatter.format(comments.get(i).getTime())  + "] " 
+								+ comments.get(i).getFullName() 
+								+ ": " 
+								+ commentTextNoHtml 
+								+ "\n";
+
+				if(commentKey != null && comments.get(i).getKey().equals(commentKey))
+					break;
+			}
+
+			return discussion;
+		}
+		catch(Exception e){
+			_log.error("Unable to reconstruct plain text discussion to put into the email body.", e);
+		}
+
+		return "";
+	}
 
 
 	/**
@@ -241,20 +413,48 @@ public class EmailPlugin {
 				notification2Save.getType() == NotificationType.MENTION ||  
 				notification2Save.getType() == NotificationType.OWN_COMMENT) {
 			String[] splits = senderEmail.split("@");
-			senderEmail = splits[0] + "+" + notification2Save.getSubjectid()  + "@" + splits[1];
+			senderEmail = splits[0] + "+" + notification2Save.getSubjectid()  + "$" + AppType.POST + "@" + splits[1];
+		}
+		if (notification2Save.getType() == NotificationType.MESSAGE) {
+			String[] splits = senderEmail.split("@");
+			senderEmail = splits[0] + "+" + notification2Save.getSubjectid()  + "$" + AppType.MSG + "@" + splits[1];
 		}
 		// EMAIL SENDER
 		msg2Return.setHeader("Content-Type", "text/html; charset=UTF-8");
 		msg2Return.setFrom(new InternetAddress(senderEmail, portalName));
 		msg2Return.addRecipient(Message.RecipientType.TO, new InternetAddress(email));
 
+		// retrieve post/comments from its id (if possible)
+		Feed feed = null;
+		List<Comment> comments = null;
+		String commentKey = null;
+
+		try{
+			String feedId = notification2Save.getSubjectid();
+			feed = store.readFeed(feedId);
+			comments = store.getAllCommentByFeed(feedId);
+			Collections.sort(comments); // sort them
+			commentKey = notification2Save.getCommentKey();
+			
+			// try to set vreName when notification is created in infrastructure scope
+			String[] splittedVREName = feed.getVreid().split("/");
+			if(vreName == null)
+				vreName = splittedVREName[splittedVREName.length - 1];
+			
+		}catch(Exception e){
+
+			_log.error("Unable to retrieve feeds/comments", e);
+
+		}
+		
+		// set subject
 		msg2Return.setSubject(getSubjectByNotificationType(notification2Save, portalUrl, vreName, user.getFirstName(), hashtags));
 
 		final MimeBodyPart textPart = new MimeBodyPart();
-		textPart.setContent(getTextEmail(notification2Save, user.getFirstName(), portalUrl, email, hashtags), "text/plain; charset=UTF-8"); 
+		textPart.setContent(getTextEmail(notification2Save, user.getFirstName(), portalUrl, email, feed, comments, commentKey, hashtags), "text/plain; charset=UTF-8"); 
 
 		final MimeBodyPart htmlPart = new MimeBodyPart();
-		htmlPart.setContent(getHTMLEmail(notification2Save, user.getFirstName(), portalUrl, email, hashtags), "text/html; charset=UTF-8");
+		htmlPart.setContent(getHTMLEmail(notification2Save, user.getFirstName(), portalUrl, email, vreName, feed, comments, commentKey, hashtags), "text/html; charset=UTF-8");
 
 		final Multipart mp = new MimeMultipart("alternative");
 		mp.addBodyPart(textPart);
@@ -274,14 +474,16 @@ public class EmailPlugin {
 	 * @param vreName
 	 * @return
 	 */
-	private static String getSubjectByNotificationType(Notification notification2Save, String portalURL, String vreName, String userFirstName, String ...hashtags) {
+	private static String getSubjectByNotificationType(Notification notification2Save, String portalURL, String vreName, String userFirstName, String ...optionalParams) {
 		switch (notification2Save.getType()) {
 		case LIKE:
-			return notification2Save.getSenderFullName()+" favorited your post";
+			return notification2Save.getSenderFullName()+" favorited your post in " + (vreName == null? "" : vreName);
 		case COMMENT:
-			return notification2Save.getSenderFullName()+" commented on the post";
+			return notification2Save.getSenderFullName()+" commented on the post in " + (vreName == null? "" : vreName);
 		case MESSAGE:
-			return notification2Save.getSenderFullName()+ " sent you a message";
+			String messageSubject = (optionalParams != null && optionalParams.length > 0) ? 
+					optionalParams[0] : notification2Save.getSenderFullName()+" sent you a message";
+					return messageSubject;
 		case WP_FOLDER_ADDEDUSER:
 			return "New user in a shared folder";
 		case WP_FOLDER_REMOVEDUSER:
@@ -303,14 +505,15 @@ public class EmailPlugin {
 		case WP_ITEM_RENAMED:
 			return notification2Save.getSenderFullName()+ " renamed an item in a shared folder";
 		case OWN_COMMENT:
-			return notification2Save.getSenderFullName() + " commented on your post";
+			return notification2Save.getSenderFullName() + " commented on your post in " + (vreName == null? "" : vreName);
 		case MENTION:
-			return notification2Save.getSenderFullName() + " mentioned you";
+			return notification2Save.getSenderFullName() + " mentioned you in " + (vreName == null? "" : vreName);
 		case POST_ALERT: 
 			String toReturn = notification2Save.getSenderFullName() + " shared a news on " + vreName;
-			if (hashtags != null)
-				for (int i = 0; i < hashtags.length; i++) 
-					toReturn += " " + hashtags[i];				
+			if (optionalParams != null) { //in this case optionalParams are the hashtags
+				for (int i = 0; i < optionalParams.length; i++) 
+					toReturn += " " + optionalParams[i];	
+			}
 			return toReturn;
 		case REQUEST_CONNECTION:
 			return "Connection request";
@@ -362,13 +565,13 @@ public class EmailPlugin {
 
 		switch (notification2Save.getType()) {
 		case LIKE:		
-			actionLink.append("\">").append(" See the Post").append("</a>");
+			actionLink.append("\">").append(" Open Post").append("</a>");
 			break;
 		case COMMENT:
-			actionLink.append("\">").append(" See the Post").append("</a>");
+			actionLink.append("\">").append(" Open Post").append("</a>");
 			break;
 		case MENTION:
-			actionLink.append("\">").append(" See the Post").append("</a>");
+			actionLink.append("\">").append(" Open Post").append("</a>");
 			break;
 		case MESSAGE:
 			actionLink.append("\">").append(" Go to Message").append("</a>");
@@ -392,7 +595,7 @@ public class EmailPlugin {
 			actionLink.append("\">").append(" Go to Folder").append("</a>");
 			break;
 		case OWN_COMMENT:
-			actionLink.append("\">").append(" See the Post").append("</a>");
+			actionLink.append("\">").append(" Open Post").append("</a>");
 			break;
 		case POST_ALERT:
 			actionLink.append("\">").append("See this News").append("</a>");
